# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - MoinSupport library (derived from EventAggregatorSupport)

    @copyright: 2008, 2009, 2010, 2011, 2012 by Paul Boddie <paul@boddie.org.uk>
    @copyright: 2000-2004 Juergen Hermann <jh@web.de>,
                2005-2008 MoinMoin:ThomasWaldmann.
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from DateSupport import *
from MoinMoin.Page import Page
from MoinMoin import wikiutil
import re
import time

__version__ = "0.1"

# Content type parsing.

encoding_regexp_str = ur'(?P<content_type>[^\s;]*)(?:;\s*charset=(?P<encoding>[-A-Za-z0-9]+))?'
encoding_regexp = re.compile(encoding_regexp_str)

# Accept header parsing.

accept_regexp_str = ur';\s*q='
accept_regexp = re.compile(accept_regexp_str)

# Utility functions.

def getContentTypeAndEncoding(content_type):

    """
    Return a tuple with the content/media type and encoding, extracted from the
    given 'content_type' header value.
    """

    m = encoding_regexp.search(content_type)
    if m:
        return m.group("content_type"), m.group("encoding")
    else:
        return None, None

def int_or_none(x):
    if x is None:
        return x
    else:
        return int(x)

# Utility classes and associated functions.

class Form:

    """
    A wrapper preserving MoinMoin 1.8.x (and earlier) behaviour in a 1.9.x
    environment.
    """

    def __init__(self, form):
        self.form = form

    def has_key(self, name):
        return not not self.form.getlist(name)

    def get(self, name, default=None):
        values = self.form.getlist(name)
        if not values:
            return default
        else:
            return values

    def __getitem__(self, name):
        return self.form.getlist(name)

class ActionSupport:

    """
    Work around disruptive MoinMoin changes in 1.9, and also provide useful
    convenience methods.
    """

    def get_form(self):
        return get_form(self.request)

    def _get_selected(self, value, input_value):

        """
        Return the HTML attribute text indicating selection of an option (or
        otherwise) if 'value' matches 'input_value'.
        """

        return input_value is not None and value == input_value and 'selected="selected"' or ''

    def _get_selected_for_list(self, value, input_values):

        """
        Return the HTML attribute text indicating selection of an option (or
        otherwise) if 'value' matches one of the 'input_values'.
        """

        return value in input_values and 'selected="selected"' or ''

    def _get_input(self, form, name, default=None):

        """
        Return the input from 'form' having the given 'name', returning either
        the input converted to an integer or the given 'default' (optional, None
        if not specified).
        """

        value = form.get(name, [None])[0]
        if not value: # true if 0 obtained
            return default
        else:
            return int(value)

def get_form(request):

    "Work around disruptive MoinMoin changes in 1.9."

    if hasattr(request, "values"):
        return Form(request.values)
    else:
        return request.form

class send_headers_cls:

    """
    A wrapper to preserve MoinMoin 1.8.x (and earlier) request behaviour in a
    1.9.x environment.
    """

    def __init__(self, request):
        self.request = request

    def __call__(self, headers):
        for header in headers:
            parts = header.split(":")
            self.request.headers.add(parts[0], ":".join(parts[1:]))

def get_send_headers(request):

    "Return a function that can send response headers."

    if hasattr(request, "http_headers"):
        return request.http_headers
    elif hasattr(request, "emit_http_headers"):
        return request.emit_http_headers
    else:
        return send_headers_cls(request)

def escattr(s):
    return wikiutil.escape(s, 1)

def getPathInfo(request):
    if hasattr(request, "getPathinfo"):
        return request.getPathinfo()
    else:
        return request.path

# Content/media type and preferences support.

class MediaRange:

    "A content/media type value which supports whole categories of data."

    def __init__(self, media_range, accept_parameters=None):
        self.media_range = media_range
        self.accept_parameters = accept_parameters or {}

        parts = media_range.split(";")
        self.media_type = parts[0]
        self.parameters = getMappingFromParameterStrings(parts[1:])

        # The media type is divided into category and subcategory.

        parts = self.media_type.split("/")
        self.category = parts[0]
        self.subcategory = "/".join(parts[1:])

    def get_parts(self):
        return self.category, self.subcategory

    def get_specificity(self):
        if "*" in self.get_parts():
            return -list(self.get_parts()).count("*")
        else:
            return len(self.parameters)

    def permits(self, other):
        if not isinstance(other, MediaRange):
            other = MediaRange(other)

        category = categoryPermits(self.category, other.category)
        subcategory = categoryPermits(self.subcategory, other.subcategory)

        if category and subcategory:
            if "*" not in (category, subcategory):
                return not self.parameters or self.parameters == other.parameters
            else:
                return True
        else:
            return False

    def __eq__(self, other):
        if not isinstance(other, MediaRange):
            other = MediaRange(other)

        category = categoryMatches(self.category, other.category)
        subcategory = categoryMatches(self.subcategory, other.subcategory)

        if category and subcategory:
            if "*" not in (category, subcategory):
                return self.parameters == other.parameters or \
                    not self.parameters or not other.parameters
            else:
                return True
        else:
            return False

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        return hash(self.media_range)

    def __repr__(self):
        return "MediaRange(%r)" % self.media_range

def categoryMatches(this, that):

    """
    Return the basis of a match between 'this' and 'that' or False if the given
    categories do not match.
    """

    return (this == "*" or this == that) and this or \
        that == "*" and that or False

def categoryPermits(this, that):

    """
    Return whether 'this' category permits 'that' category. Where 'this' is a
    wildcard ("*"), 'that' should always match. A value of False is returned if
    the categories do not otherwise match.
    """

    return (this == "*" or this == that) and this or False

def getMappingFromParameterStrings(l):

    """
    Return a mapping representing the list of "name=value" strings given by 'l'.
    """

    parameters = {}

    for parameter in l:
        parts = parameter.split("=")
        name = parts[0].strip()
        value = "=".join(parts[1:]).strip()
        parameters[name] = value

    return parameters

def getContentPreferences(accept):

    """
    Return a mapping from media types to parameters for content/media types
    extracted from the given 'accept' header value. The mapping is returned in
    the form of a list of (media type, parameters) tuples.

    See: http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.1
    """

    preferences = []

    for field in accept.split(","):

        # The media type with parameters (defined by the "media-range") is
        # separated from any other parameters (defined as "accept-extension"
        # parameters) by a quality parameter.

        fparts = accept_regexp.split(field)

        # The first part is always the media type.

        media_type = fparts[0].strip()

        # Any other parts can be interpreted as extension parameters.

        if len(fparts) > 1:
            fparts = ("q=" + ";q=".join(fparts[1:])).split(";")
        else:
            fparts = []

        # Each field in the preferences can incorporate parameters separated by
        # semicolon characters.

        parameters = getMappingFromParameterStrings(fparts)
        media_range = MediaRange(media_type, parameters)
        preferences.append(media_range)

    return ContentPreferences(preferences)

class ContentPreferences:

    "A wrapper around content preference information."

    def __init__(self, preferences):
        self.preferences = preferences

    def __iter__(self):
        return iter(self.preferences)

    def get_ordered(self, by_quality=0):

        """
        Return a list of content/media types in descending order of preference.
        If 'by_quality' is set to a true value, the "q" value will be used as
        the primary measure of preference; otherwise, only the specificity will
        be considered.
        """

        ordered = {}

        for media_range in self.preferences:
            specificity = media_range.get_specificity()

            if by_quality:
                q = float(media_range.accept_parameters.get("q", "1"))
                key = q, specificity
            else:
                key = specificity

            if not ordered.has_key(key):
                ordered[key] = []

            ordered[key].append(media_range)

        # Return the preferences in descending order of quality and specificity.

        keys = ordered.keys()
        keys.sort(reverse=True)
        return [ordered[key] for key in keys]

    def get_preferred_type(self, available):

        """
        Return the preferred content/media type from those in the 'available'
        list, given the known preferences.
        """

        matches = {}
        available = set(available[:])

        for level in self.get_ordered():
            for media_range in level:

                # Attempt to match available types.

                found = set()
                for available_type in available:
                    if media_range.permits(available_type):
                        q = float(media_range.accept_parameters.get("q", "1"))
                        if not matches.has_key(q):
                            matches[q] = []
                        matches[q].append(available_type)
                        found.add(available_type)

                # Stop looking for matches for matched available types.

                if found:
                    available.difference_update(found)

        # Sort the matches in descending order of quality.

        all_q = matches.keys()

        if all_q:
            all_q.sort(reverse=True)
            return matches[all_q[0]]
        else:
            return None

# Page access functions.

def getPageURL(page):

    "Return the URL of the given 'page'."

    request = page.request
    return request.getQualifiedURL(page.url(request, relative=0))

def getFormat(page):

    "Get the format used on the given 'page'."

    return page.pi["format"]

def getMetadata(page):

    """
    Return a dictionary containing items describing for the given 'page' the
    page's "created" time, "last-modified" time, "sequence" (or revision number)
    and the "last-comment" made about the last edit.
    """

    request = page.request

    # Get the initial revision of the page.

    revisions = page.getRevList()
    event_page_initial = Page(request, page.page_name, rev=revisions[-1])

    # Get the created and last modified times.

    initial_revision = getPageRevision(event_page_initial)

    metadata = {}
    metadata["created"] = initial_revision["timestamp"]
    latest_revision = getPageRevision(page)
    metadata["last-modified"] = latest_revision["timestamp"]
    metadata["sequence"] = len(revisions) - 1
    metadata["last-comment"] = latest_revision["comment"]

    return metadata

def getPageRevision(page):

    "Return the revision details dictionary for the given 'page'."

    # From Page.edit_info...

    if hasattr(page, "editlog_entry"):
        line = page.editlog_entry()
    else:
        line = page._last_edited(page.request) # MoinMoin 1.5.x and 1.6.x

    # Similar to Page.mtime_usecs behaviour...

    if line:
        timestamp = line.ed_time_usecs
        mtime = wikiutil.version2timestamp(long(timestamp)) # must be long for py 2.2.x
        comment = line.comment
    else:
        mtime = 0
        comment = ""

    # Leave the time zone empty.

    return {"timestamp" : DateTime(time.gmtime(mtime)[:6] + (None,)), "comment" : comment}

# User interface functions.

def getParameter(request, name, default=None):

    """
    Using the given 'request', return the value of the parameter with the given
    'name', returning the optional 'default' (or None) if no value was supplied
    in the 'request'.
    """

    return get_form(request).get(name, [default])[0]

def getQualifiedParameter(request, prefix, argname, default=None):

    """
    Using the given 'request', 'prefix' and 'argname', retrieve the value of the
    qualified parameter, returning the optional 'default' (or None) if no value
    was supplied in the 'request'.
    """

    argname = getQualifiedParameterName(prefix, argname)
    return getParameter(request, argname, default)

def getQualifiedParameterName(prefix, argname):

    """
    Return the qualified parameter name using the given 'prefix' and 'argname'.
    """

    if prefix is None:
        return argname
    else:
        return "%s-%s" % (prefix, argname)

# Page-related functions.

def getPrettyPageName(page):

    "Return a nicely formatted title/name for the given 'page'."

    title = page.split_title(force=1)
    return getPrettyTitle(title)

def linkToPage(request, page, text, query_string=None):

    """
    Using 'request', return a link to 'page' with the given link 'text' and
    optional 'query_string'.
    """

    text = wikiutil.escape(text)
    return page.link_to_raw(request, text, query_string)

def linkToResource(url, request, text, query_string=None):

    """
    Using 'request', return a link to 'url' with the given link 'text' and
    optional 'query_string'.
    """

    if query_string:
        query_string = wikiutil.makeQueryString(query_string)
        url = "%s?%s" % (url, query_string)

    formatter = request.page and getattr(request.page, "formatter", None) or request.html_formatter

    output = []
    output.append(formatter.url(1, url))
    output.append(formatter.text(text))
    output.append(formatter.url(0))
    return "".join(output)

def getFullPageName(parent, title):

    """
    Return a full page name from the given 'parent' page (can be empty or None)
    and 'title' (a simple page name).
    """

    if parent:
        return "%s/%s" % (parent.rstrip("/"), title)
    else:
        return title

# vim: tabstop=4 expandtab shiftwidth=4
